use crate::prelude::*;
use crate::samplers::sampler::Sampler3D;

pub struct SplineSampler3D {
    lower_end: Vector3,
    lower_control: Vector3,
    upper_control: Vector3,
    upper_end: Vector3,
    bake_interval: f32,
}

impl Sampler3D {
    fn sample(&self, t: f32) -> Vector3 {
        let q0 = lower_end + lower_control * t;
        let q1 = (lower_end + lower_control).lerp((upper_end + upper_control), t);
        let q2 = upper_end + upper_control * (1-t);
        
        let r0 = q0.lerp(q1, t);
        let r1 = q1.lerp(q2, t);
        
        return r0.lerp(r1, t);
    }
    
    fn tangent(&self, t: f32) -> Vector3 {
        let q0 = lower_end + lower_control * t;
        let q1 = (lower_end + lower_control).lerp((upper_end + upper_control), t);
        let q2 = upper_end + upper_control * (1-t);
        
        let r0 = q0.lerp(q1, t);
        let r1 = q1.lerp(q2, t);
        
        return r1 - r0;
    }
}
